/**
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 2007 Bull S.A.S.
 * Contact: jonas-team@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 *
 * --------------------------------------------------------------------------
 * $Id: DeployableMonitorService.java 10853 2007-07-05 14:39:33Z benoitf $
 * --------------------------------------------------------------------------
 */

package org.ow2.jonas.deployablemonitor;

import java.io.File;
import java.util.LinkedList;
import java.util.List;

import javax.naming.Context;
import javax.naming.NamingException;

import org.objectweb.jonas.common.JProp;
import org.objectweb.jonas.service.AbsServiceImpl;
import org.objectweb.jonas.service.Service;
import org.objectweb.jonas.service.ServiceException;
import org.ow2.jonas.deployer.DeployerManagerService;
import org.ow2.util.log.Log;
import org.ow2.util.log.LogFactory;

/**
 * Service that will launch the Deployable Monitor.
 * @author Florent BENOIT
 */
public class DeployableMonitorService extends AbsServiceImpl implements Service {

    /**
     * Property for the development mode.
     */
    public static final String DEVELOPMENT_MODE_PROPERTY = "development";

    /**
     * Property for the list of extra directories to analyze.
     */
    public static final String DIRECTORIES_LIST_PROPERTY = "directories";

    /**
     * The name of the JONAS_BASE directory.
     */
    protected static final String JONAS_BASE = JProp.getJonasBase();

    /**
     * Default directory to analyze.
     */
    public static final String DEFAULT_DIRECTORY = JONAS_BASE + File.separator + "deploy";

    /**
     * Logger.
     */
    private Log logger = LogFactory.getLog(DeployableMonitor.class);

    /**
     * Development mode or not ? Default is true.
     */
    private boolean developmentMode = true;

    /**
     * List of directories.
     */
    private List<File> directories = null;


    /**
     * Link to the deployable monitor.
     */
    private DeployableMonitor deployableMonitor = null;

    /**
     * Default constructor.
     */
    public DeployableMonitorService() {
        this.directories = new LinkedList<File>();
        this.deployableMonitor = new DeployableMonitor();
    }

    /**
     * Init the Deployable Monitor service.
     * @param ctx the configuration of the Deployable Monitor service.
     * @throws ServiceException if the initialization failed.
     */
    protected void doInit(final Context ctx) throws ServiceException {

        // Development mode
        String inDevStr = "true";
        try {
            inDevStr = (String) ctx.lookup(DEVELOPMENT_MODE_PROPERTY);
        } catch (NamingException e) {
            logger.warn("Property ''{0}'' is missing, Develomment mode is set to true by default.", DEVELOPMENT_MODE_PROPERTY);
        }
        this.developmentMode = Boolean.parseBoolean(inDevStr);

        // Extra directories to scan.
        String configDirectoriesStr = null;
        try {
            configDirectoriesStr = (String) ctx.lookup(DIRECTORIES_LIST_PROPERTY);
        } catch (NamingException e) {
            logger.warn("Property ''{0}'' is missing, No extra directories to analyze.", DIRECTORIES_LIST_PROPERTY);
        }

        // First, add the default directory
        File defaultDirectory = new File(DEFAULT_DIRECTORY);
        if (!defaultDirectory.exists()) {
            logger.info("Creating default deploy directory ''{0}''", defaultDirectory);
            defaultDirectory.mkdirs();
        }
        directories.add(defaultDirectory);

        // Use String Tokenizer
        if (configDirectoriesStr != null && configDirectoriesStr.length() > 0) {
            String[] additionalDirectories = configDirectoriesStr.split(",");
            for (String dir : additionalDirectories) {
                File tmpFile = new File(dir);

                // Not an absolute file, should be relative to JONAS_BASE.
                if (!tmpFile.isAbsolute()) {
                    tmpFile = new File(JONAS_BASE + File.separator + dir);
                }

                // Directory exists ?
                if (!tmpFile.exists()) {
                    logger.warn("The given directory ''{0}'' is neither present on the filesystem or in JONAS_BASE ''{1}''",
                            tmpFile, JONAS_BASE);
                    break;
                }

                // Add the directory
                directories.add(tmpFile);

            }
        }

        logger.info("Use the deploy directories ''{0}'', development mode is ''{1}''", directories, developmentMode);

        // configure it
        deployableMonitor.setDevelopmentMode(developmentMode);
        deployableMonitor.setDirectories(directories);
        deployableMonitor.setDeployerManager(DeployerManagerService.getDeployerManager());

    }

    /**
     * Start the Deployable Monitor service.
     * @throws ServiceException if the startup failed.
     */
    protected void doStart() throws ServiceException {
        deployableMonitor.start();
    }

    /**
     * Stop the Deployable Monitor service.
     * @throws ServiceException if the stop failed.
     */
    protected void doStop() throws ServiceException {
        deployableMonitor.stopOrder();
    }
}
