/**
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999-2007 Bull S.A.S.
 * Contact: jonas-team@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id:RegistryServiceImpl.java 10372 2007-05-14 13:58:42Z sauthieg $
 * --------------------------------------------------------------------------
 */

package org.objectweb.jonas.registry.carol;

import java.net.URI;
import java.util.ArrayList;
import java.util.List;

import javax.naming.Context;
import javax.naming.NamingException;

import org.objectweb.carol.jndi.ns.NameServiceManager;
import org.objectweb.carol.util.configuration.CarolDefaultValues;
import org.objectweb.carol.util.configuration.ConfigurationRepository;
import org.objectweb.carol.util.configuration.Protocol;
import org.objectweb.carol.util.configuration.ProtocolConfiguration;
import org.objectweb.jonas.naming.JComponentContextFactory;
import org.objectweb.jonas.naming.JComponentContextFactoryDelegate;
import org.objectweb.jonas.naming.context.ComponentContext;
import org.objectweb.jonas.naming.context.SingletonComponentContextFactory;
import org.objectweb.jonas.registry.RegistryService;
import org.objectweb.jonas.registry.carol.delegate.ORBCCFDelegate;
import org.objectweb.jonas.service.AbsServiceImpl;
import org.objectweb.jonas.service.ServiceException;

/**
 * Implementation of the {@link RegistryService} interface.
 * @author Helene Joanin Contributor(s): Julien Lehembre (Libelis)
 */
public class CarolRegistryService extends AbsServiceImpl implements RegistryService {

    /**
     * Registry mode property name.
     */
    public static final String MODE = "jonas.service.registry.mode";

    /**
     * Registry class property name.
     */
    public static final String CLASS = "jonas.service.registry.class";

    /**
     * Automatic mode.
     */
    public static final String DEFAULT_MODE = "automatic";

    /**
     * Collocated mode.
     */
    public static final String COLLOCATED = "collocated";

    /**
     * Remote mode.
     */
    public static final String REMOTE = "remote";

    /**
     * Should the local registry be started ?
     */
    private boolean bStartRegistry = true;

    /**
     * Should this service ignore registry startup errors ?
     */
    private boolean bIgnoreError = true;

    /**
     * The ORB Delegate.
     */
    private JComponentContextFactoryDelegate delegate = null;

    /**
     * {@link ComponentContext} factory.
     */
    private JComponentContextFactory factory = null;

    public void doInit(Context ctx) {

        String propRegistry = null;
        try {
            propRegistry = (String) ctx.lookup(MODE);
        } catch (NamingException e) {
            // No problem if there is no value --> default value
            propRegistry = DEFAULT_MODE;
        }

        if (DEFAULT_MODE.equalsIgnoreCase(propRegistry)) {
            bStartRegistry = true;
            bIgnoreError = true;
        } else if (COLLOCATED.equalsIgnoreCase(propRegistry)) {
            bStartRegistry = true;
            bIgnoreError = false;
        } else if (REMOTE.equalsIgnoreCase(propRegistry)) {
            bStartRegistry = false;
        }

        delegate = new ORBCCFDelegate();

    }

    public void doStart() throws ServiceException {
        if (bStartRegistry) {
            try {
                if (bIgnoreError) {
                    // start the carol name service manager
                    NameServiceManager.startNonStartedNS();
                } else {
                    NameServiceManager.startNS();
                }
            } catch (Exception e) {
                throw new ServiceException("JOnAS: Cannot start the registry", e);
            }
        }

        // Init the ComponentContextFactory
        try {
            factory = SingletonComponentContextFactory.getInstance();
        } catch (NamingException e) {
            throw new ServiceException("Cannot initialize the ComponentContextFactory", e);
        }

        // Add delegate
        try {
            factory.addDelegate(delegate);
        } catch (NamingException e) {
            throw new ServiceException("Cannot add the delegate", e);
        }
    }

    public void doStop() throws ServiceException {

        // Remove the delegate
        try {
            factory.removeDelegate(delegate);
        } catch (NamingException e) {
            throw new ServiceException("Cannot remove the delegate", e);
        }

        try {
            // Stop the carol name service manager
            NameServiceManager.stopNS();
        } catch (Exception e) {
            throw new ServiceException("Problem when stopping registry service", e);
        }
    }

    /**
     * {@inheritDoc}
     */
    public List getActiveProtocolNames() {
        List protocols = new ArrayList();

        ProtocolConfiguration[] configs = ConfigurationRepository.getConfigurations();
        for (int i = 0; i < configs.length; i++) {
            protocols.add(configs[i].getName());
        }
        return protocols;
    }

    /**
     * {@inheritDoc}
     */
    public String getDefaultProtocolName() {
        return ConfigurationRepository.getDefaultConfiguration().getName();
    }

    /**
     * {@inheritDoc}
     */
    public int getExportedObjectPort(String protocolName) {
        ProtocolConfiguration config = getProtocolConfiguration(protocolName);
        String key = CarolDefaultValues.CAROL_PREFIX + protocolName + ".server.port";
        String port = config.getProperties().getProperty(key, "0");
        return Integer.valueOf(port);
    }

    /**
     * {@inheritDoc}
     */
    public String getInitialContextFactoryName(String protocolName) {
        ProtocolConfiguration config = getProtocolConfiguration(protocolName);
        return config.getProtocol().getInitialContextFactoryClassName();
    }

    /**
     * {@inheritDoc}
     */
    public URI getProviderURL(String protocolName) {
        ProtocolConfiguration config = getProtocolConfiguration(protocolName);
        return URI.create(config.getProviderURL());
    }

    /**
     * {@inheritDoc}
     */
    public void setDefaultProtocol(String protocolName) {
        ProtocolConfiguration config = getProtocolConfiguration(protocolName);
        ConfigurationRepository.setCurrentConfiguration(config);
    }

    /**
     * @param protocolName parotocol name.
     * @return Returns the {@link Protocol} with the given name.
     */
    private ProtocolConfiguration getProtocolConfiguration(String protocolName) {
        return ConfigurationRepository.getConfiguration(protocolName);
    }

}
