/**
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 2007 Bull S.A.S.
 * Contact: jonas-team@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: UserTransactionCCFDelegate.java 10634 2007-06-13 16:32:27Z benoitf $
 * --------------------------------------------------------------------------
 */

package org.objectweb.jonas.jtm.jotm.delegate;

import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;

import org.objectweb.jonas.common.Log;
import org.objectweb.jonas.naming.JComponentContextFactoryDelegate;
import org.objectweb.util.monolog.api.BasicLevel;
import org.objectweb.util.monolog.api.Logger;

/**
 * Store the {@link javax.transaction.UserTransaction} UserTransaction object
 * under the <code>java:comp/UserTransaction</code> name.
 * @author Guillaume Sauthier
 */
public class UserTransactionCCFDelegate implements JComponentContextFactoryDelegate {

    /**
     * UT Context Name (in java:comp/).
     */
    private static final String USER_TRANSACTION = "UserTransaction";

    /**
     * JNDI Global name for the UserTransaction.
     */
    private static final String USER_TRANSACTION_JNDI_NAME = "javax.transaction.UserTransaction";

    /**
     * Logger used for traces.
     */
    private static Logger logger = Log.getLogger(Log.JONAS_NAMING_PREFIX);

    /**
     * UserTransaction object, to be shared by all components.
     */
    private Object userTransaction = null;

    /**
     * Base InitialContext.
     */
    private InitialContext ictx;

    /**
     * Initialize the Context.
     * @throws NamingException if unable to create the InitialContext.
     */
    private void init() throws NamingException {
        // Initial Context
        ictx = new InitialContext();

        // Get userTransaction instance
        try {
            userTransaction = ictx.lookup(USER_TRANSACTION_JNDI_NAME);
        } catch (NamingException ne) {
            if (Log.isDebugNaming()) {
                logger.log(BasicLevel.DEBUG, "Cannot lookup " + USER_TRANSACTION_JNDI_NAME + ".");
            }
        }

    }

    /**
     * {@inheritDoc}
     */
    public void modify(final Context componentContext) throws NamingException {

        // init if needed
        if (userTransaction == null) {
            init();
        }

        // Bind java:comp/UserTransaction
        if (userTransaction != null) {
            componentContext.rebind(USER_TRANSACTION, userTransaction);
        }
    }

    /**
     * Undo the changes done by this delegate on the given java:comp context.
     * @param componentContext <code>java:comp/</code> component context to be modified.
     * @throws NamingException thrown if something goes wrong
     *         during the {@link javax.naming.Context} update.
     */
    public void undo(final Context componentContext) throws NamingException {
        if (userTransaction != null) {
            componentContext.unbind(USER_TRANSACTION);
        }
    }

}
