/**
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 2007 Bull S.A.S
 * Contact: jonas-team@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: JOnASWorkManagerService.java 10454 2007-05-24 11:39:18Z sauthieg $
 * --------------------------------------------------------------------------
 */

package org.objectweb.jonas.workmanager.internal;

import javax.management.ObjectName;
import javax.naming.Context;
import javax.naming.NamingException;
import javax.resource.spi.work.WorkManager;

import org.objectweb.jonas.common.Log;
import org.objectweb.jonas.jmx.JmxService;
import org.objectweb.jonas.jmx.oname.JonasObjectName;
import org.objectweb.jonas.jtm.TransactionService;
import org.objectweb.jonas.service.AbsServiceImpl;
import org.objectweb.jonas.service.ServiceException;
import org.objectweb.jonas.service.manager.ServiceManager;
import org.objectweb.jonas.workmanager.WorkManagerService;
import org.objectweb.util.monolog.api.BasicLevel;
import org.objectweb.util.monolog.api.Logger;

/**
 * The JOnAS {@link WorkManager} is returned by this service.
 * @author Guillaume Sauthier
 */
public class JOnASWorkManagerService extends AbsServiceImpl implements WorkManagerService {

    /**
     * Logger for this service.
     */
    private static Logger logger = null;

    /**
     * The JOnAS {@link WorkManager} implementation.
     */
    private JWorkManager workManager = null;

    // Configuration properties -----------------------

    /**
     * Minimum pool size.
     */
    private int minPoolSize;

    /**
     * Maximum pool size.
     */
    private int maxPoolSize;

    /**
     * Thread wait timeout.
     */
    private long threadWaitingTime;

    // Service dependencies -------------------------

    /**
     * JMX Services for MBeans registration.
     */
    private JmxService jmx;

    /**
     * {@link TransactionService} service.
     */
    private TransactionService tmService;

    // ------------------------------------------------------------
    // WorkManager thread pool parameters names and default values
    // ------------------------------------------------------------
    /** Min Work Thread property name */
    public static final String MIN_WORK_THREADS = "jonas.service.wm.minworkthreads";

    /** Min Work Thread default value */
    private static final int MIN_WORK_THREADS_DEFAULT = 3;

    /** Max Work Thread property name */
    public static final String MAX_WORK_THREADS = "jonas.service.wm.maxworkthreads";

    /** Max Work Thread default value */
    private static final int MAX_WORK_THREADS_DEFAULT = 80;

    /** Thread Wait Timeout property name */
    public static final String THREAD_WAIT_TIMEOUT = "jonas.service.wm.threadwaittimeout";

    /** Thread Wait Timeout default value */
    private static final int THREAD_WAIT_TIMEOUT_DEFAULT = 60;

    @Override
    protected void doInit(final Context ctx) throws ServiceException {
        // Get the service logger
        logger = Log.getLogger(Log.JONAS_WORK_MGR_PREFIX);

        // Retrieve configuration
        minPoolSize = getContextParameter(ctx, MIN_WORK_THREADS, MIN_WORK_THREADS_DEFAULT);
        maxPoolSize = getContextParameter(ctx, MAX_WORK_THREADS, MAX_WORK_THREADS_DEFAULT);
        threadWaitingTime = getContextParameter(ctx, THREAD_WAIT_TIMEOUT, THREAD_WAIT_TIMEOUT_DEFAULT);

        // Dependencies
        //----------------------------------------
        try {
            // Get the JMX Service
            ServiceManager sm = ServiceManager.getInstance();
            jmx = (JmxService) sm.getJmxService();
        } catch (ServiceException e) {
            logger.log(BasicLevel.ERROR, "Cannot initialize the JMX Service");
            throw new ServiceException("Can't get access to the JMX service", e);
        }

        try {
            // Get the TM Service
            ServiceManager sm = ServiceManager.getInstance();
            tmService = (TransactionService) sm.getTransactionService();
        } catch (ServiceException e) {
            logger.log(BasicLevel.ERROR, "Can't get access to the TransactionService");
            throw new ServiceException("Can't get access to the TransactionService", e);
        }

    }

    /**
     * @param ctx Context used to get values.
     * @param parameter parameter name
     * @param defaultValue typed default value
     * @return Returns the parameter value found in {@link Context}, or defaultValue if not found.
     */
    private static int getContextParameter(final Context ctx,
                                           final String parameter,
                                           final int defaultValue) {
        int value;
        try {
            String stringified = (String) ctx.lookup(parameter);
            value = new Integer(stringified).intValue();
        } catch (NamingException e) {
            // if not found default value will be used.
            value = defaultValue;
        }
        return value;
    }

    @Override
    protected void doStart() throws ServiceException {
        // Instanciate the WorkManager
        workManager = new JWorkManager(minPoolSize,
                                       maxPoolSize,
                                       tmService,
                                       threadWaitingTime);

        // register the MBean
        registerWorkManagerMBean(getDomainName(), workManager);

        logger.log(BasicLevel.INFO, "WorkManager Service started.");
    }

    @Override
    protected void doStop() throws ServiceException {
        // stop WorkManager threads
        workManager.stopThreads();
        workManager = null;

        // unregister the MBean
        unregisterWorkManagerMBean(getDomainName());

        logger.log(BasicLevel.INFO, "WorkManager Service stopped.");
    }

    /**
     * {@inheritDoc}
     */
    public WorkManager getWorkManager() {
        return workManager;
    }

    /**
     * Create an MBean for the JOnAS WorkManager
     * @param domainName domain name
     * @param workManager JOnAS WorkManager to be managed
     */
    private void registerWorkManagerMBean(final String domainName,
                                          final JWorkManager workManager) {
        // Create and register the MBean for this WorkManager
        ObjectName on = JonasObjectName.workManager(domainName);
        try {
            jmx.registerModelMBean(workManager, on.toString());
        } catch (Exception e) {
            logger.log(BasicLevel.WARN, "Could not register WorkManager MBean", e);
        }
    }


    private void unregisterWorkManagerMBean(final String domainName) {
        ObjectName on = JonasObjectName.workManager(domainName);
        try {
            jmx.unregisterModelMBean(on);
        } catch (Exception e) {
            logger.log(BasicLevel.WARN, "Could not unregister WorkManager MBean", e);
        }
    }
}
